/*****************************************************************************
 *                                                                           *
 *                Licensed Materials - Property of IBM Corp.                 *
 *                  5648-C60 (c) Copyright IBM Corp. 2000                    *
 *                                                                           *
 *       MQJMS Standalone Application Server Facilities (ASF) Samples        *
 *                                                                           *
 *                                                                           *
 *  FILENAME:     Load1.java                                                 *
 *                                                                           *
 *  DESCRIPTION:  This application simply loads an MQ queue with messages,   *
 *                and can be used to exercise the ASF functionality within   *
 *                MQJMS.                                                     *
 *                                                                           *
 *                Further information about this class and its usage can be  *
 *                found in the 'Websphere MQ Using Java' documentation.      *
 *                                                                           *
 *  USAGE:        java Load1 -nojndi [-qmgr <qMgrName>] [-q <qName>]           *
 *             or                                                            *
 *                java Load1 [-icf <jndiICF>] [-url <jndiURL>]               *
 *                           [-qcflookup <qcfLookup>] [-qlookup <qLookup>]   *
 *                                                                           *
 *                Both forms can additionally take the following arguments:  *
 *                          [-sleep <sleepTime>] [-msgs <numMsgs>]           *
 *                                                                           *
 *                Defaults for these parameters can be found below.          *
 *                                                                           *
 *  HISTORY:      010131 - First released as part of SupportPac MA88 v2      *
 *                                                                           *
 *****************************************************************************/

import javax.jms.*;
import javax.naming.*;
import java.util.Hashtable;

public class Load1 {
   // These are the defaults for the parameters which can be over-ridden
   // on the command line

   // Will we use JNDI for administered object creation?
   private static boolean useJNDI   = true;
   // JNDI parameters
   private static String  qcfLookup = "cn=qcf";
   private static String  qLookup   = "cn=q";
   private static String  icf       = "com.sun.jndi.ldap.LdapCtxFactory";
   private static String  url       = "ldap://localhost/o=ibm,c=us";
   // Non-JNDI parameters
   private static String  qMgrName  = "";
   private static String  qName     = "SYSTEM.DEFAULT.LOCAL.QUEUE";
   // Other parameters
   private static int     numMsgs   = 1000;
   private static int     sleepTime = 0;        // Default is to not sleep
                                                // between puts

   /**************************************************************************
    *  main - Application entry point                                        *
    **************************************************************************/
   public static void main( String[] args ) throws JMSException {
      // Parse the command line
      parseArguments(args);

      // Obtain the two administered objects we need
      Object[] adminObjects = new Object[2];
      obtainAdminObjects( adminObjects );
      QueueConnectionFactory qcf = (QueueConnectionFactory)adminObjects[0];
      Queue q = (Queue)adminObjects[1];

      // Create a QueueConnection and start it
      QueueConnection conn = qcf.createQueueConnection();
      conn.start();

      // Create a QueueSession
      QueueSession sess = conn.createQueueSession( false,
                                                   Session.AUTO_ACKNOWLEDGE );
      // Get a sender
      QueueSender sender = sess.createSender(q);

      // Display the destination we are using (requires access to MQJMS-
      // specific properties)
      System.out.println("\nTarget Queue Manager : '"+
                  ((com.ibm.mq.jms.MQQueue)q).getBaseQueueManagerName()+"'");
      System.out.println("Target Queue         : '"+
                  ((com.ibm.mq.jms.MQQueue)q).getBaseQueueName()+"'");

      // Create a message to repeatedly send
      TextMessage msg = sess.createTextMessage();
      msg.setText("A message from the 'Load1' sample application");

      System.out.print("Sending messages     : ");

      // Send the message 'numMsgs' times, with a sleep of 'sleep' ms between
      // each send (if 'sleep' is zero, do not sleep between sends)
      for( int i=0; i<numMsgs; i++ ) {
         sender.send(msg);

         if( i%(numMsgs/10)==0 ) System.out.print(".");

         if( sleepTime!=0 ) {
            try {
               Thread.sleep(sleepTime);
            } catch( InterruptedException ie ) {
               // Ignore it
            }
         }
      }

      // Finally close the QueueConnection and quit
      conn.close();
      System.out.println(" Done!");
   }

   /**************************************************************************
    *  parseArguments - Parse the given argument array, updating member      *
    *                   variables as we go. Terminates the application if an *
    *                   error is encountered.                                *
    **************************************************************************/
   private static void parseArguments( String[] args ) {
      try {
         for( int i=0; i<args.length; i++ ) {
            String arg = args[i].toUpperCase();

            if( arg.equals("-NOJNDI"))         useJNDI   = false;
            else if( arg.equals("-URL"))       url       = args[++i];
            else if( arg.equals("-ICF"))       icf       = args[++i];
            else if( arg.equals("-QCFLOOKUP")) qcfLookup = args[++i];
            else if( arg.equals("-QLOOKUP"))   qLookup   = args[++i];
            else if( arg.equals("-Q"))         qName     = args[++i];
            else if( arg.equals("-QMGR"))      qMgrName  = args[++i];
            else if( arg.equals("-MSGS"))      numMsgs   = Integer.parseInt(
                                                                   args[++i]);
            else if( arg.equals("-SLEEP"))     sleepTime = Integer.parseInt(
                                                                   args[++i]);
            else
               throw new Exception("Unrecognised argument ("+arg+")");
         }
      } catch( Exception e ) {
         System.out.println("Error parsing arguments: "+e+"\nQuitting.");
         System.exit(-1);
      }
   }

   /**************************************************************************
    *  obtainAdminObjects - Obtains the two administered objects we need and *
    *                       places references to them in the given array.    *
    *                       The method used (JNDI versus explicit construct- *
    *                       ion) is determined by the value of 'useJNDI'.    *
    *                                                                        *
    *                       The application will terminate if any problems   *
    *                       are encountered.                                 *
    **************************************************************************/
   private static void obtainAdminObjects( Object[] outArray ) {
      try {
         if( useJNDI ) {
            // Set up JNDI environment parameters
            Hashtable env = new Hashtable();
            env.put( Context.PROVIDER_URL, url );
            env.put( Context.INITIAL_CONTEXT_FACTORY, icf );

            // Access the JNDI namespace
            Context ctx = new InitialContext(env);

            // Obtain the objects
            outArray[0] = ctx.lookup(qcfLookup);
            outArray[1] = ctx.lookup(qLookup);

         } else {
            // Create the objects explicitly - we need to refer to the
            // MQJMS-specific classes now
            com.ibm.mq.jms.MQQueueConnectionFactory qcf =
                                 new com.ibm.mq.jms.MQQueueConnectionFactory();
            qcf.setQueueManager(qMgrName);

            com.ibm.mq.jms.MQQueue q = new com.ibm.mq.jms.MQQueue();
            q.setBaseQueueName(qName);
            q.setBaseQueueManagerName(qMgrName);

            outArray[0] = qcf;
            outArray[1] = q;
         }
      } catch( Exception e ) {
         System.out.println("Error obtaining admin objects: "+e+"\nQuitting.");
         System.exit(-1);
      }
   }
}
